/**
 * @date 06.08.2009
 * @author uzi
 */
package omdb;

import java.io.IOException;
import java.net.URLEncoder;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;

/**
 * @author uzi
 * @date 06.08.2009
 */
public class OmdbConnection
{
    /**
     * language de. use with setLanguage().
     */
    public static final byte DE = 1;
    
    /**
     * language en. use with setLanguage().
     */
    public static final byte EN = 2;
    
    
    
    /**
     * sets the language of the session to de
     */
    private static final String LANGUAGE_URL_DE = "http://www.omdb.org/session/set_language?language=1556";
    
    /**
     * sets the language of the session to en
     */
    private static final String LANGUAGE_URL_EN = "http://www.omdb.org/session/set_language?language=1819";
    
    /**
     * url to save the abscract. %d will be replaced by the movie id.
     */
    private static final String SET_ABSTRACT_URL = "http://www.omdb.org/movie/%d/set_abstract";
    
    
    /**
     * the http client holds the http session und simplifies the communication.
     */
    private HttpClient httpClient = new HttpClient();
    
    
    
    /**
     * sets the language of this session. the default language for omdb is english. so every
     * data saved will be used for the english sub-page of the movie. if you want to save 
     * german data, eg an abstract, call this method first to set the language to de.
     * 
     * the method is blocking, so it returns, as soon as omdb sends the http response but
     * not before.
     * 
     * @param language OmdbConnection.EN or OmdbConnection.DE
     * @return true if the response code was 200 (ok), false otherwise or if the language is not supported
     * @throws IOException if the communication with omdb failed
     */
    public boolean setLanguage(byte language) 
    throws IOException
    {
        GetMethod getMethod;
        if (language == OmdbConnection.DE)
        {
            getMethod = new GetMethod(OmdbConnection.LANGUAGE_URL_DE);
        }
        else if (language == OmdbConnection.EN)
        {
            getMethod = new GetMethod(OmdbConnection.LANGUAGE_URL_EN);
        }
        else
        {
            //unsupported language
            return false;
        }
        int statusCode = httpClient.executeMethod(getMethod);
        return (statusCode == 200);
    }
    
    
    /**
     * saves an abstract for a movie. the language is controlled via setLanguage.
     * the abstract may not have more than 400 characters.
     * 
     * the method is blocking, so it returns, as soon as omdb sends the http response but
     * not before.
     * 
     * @param movieId the omdb-id of the movie
     * @param movieAbstract the abstract
     * @return true if the response code was 200 (ok), false otherwise
     * @throws IOException if the communication with omdb failed
     */
    public boolean saveAbstract(long movieId, String movieAbstract) 
    throws IOException
    {
        PostMethod postMethod = new PostMethod(String.format(OmdbConnection.SET_ABSTRACT_URL, movieId));
        postMethod.setRequestEntity(new StringRequestEntity("movie%5Babstract%5D=" + URLEncoder.encode(movieAbstract, "UTF-8") + "&commit=Ok", "application/x-www-form-urlencoded; charset=UTF-8", "UTF-8"));
        
        //omdb need this headers, it wont work without them
        postMethod.setRequestHeader("X-Requested-With", "XMLHttpRequest");
        postMethod.setRequestHeader("X-Prototype-Version", "1.5.0");
        
        int statusCode = httpClient.executeMethod(postMethod);
        return (statusCode == 200);
    }
    
    
    public static void main(String[] args)
    throws Exception
    {
        OmdbConnection omdb = new OmdbConnection();
        
        if (omdb.setLanguage(OmdbConnection.DE))
        {
            System.out.println(omdb.saveAbstract(12287, "test foo de"));
        }
        else
        {
            System.out.println("false");
        }
    }
}
